@@ -4,23 +4,24 @@ The `servo.repeating.Mixin` provides connectors with the ability to easily manag
 that require periodic execution or the observation of particular runtime conditions.
 """
 import asyncio
-import datetime
-import weakref
 from typing import Callable, Dict, Optional, Union
 
-import servo.utilities.key_paths
+import pydantic
+
 from servo.types import Duration, NoneCallable, Numeric
 
 __all__ = ["Every", "Mixin", "repeating"]
 
 Every = Union[Numeric, str, Duration]
 
-_repeating_tasks_registry = weakref.WeakKeyDictionary()
 
-
-class Mixin:
+class Mixin(pydantic.BaseModel):
     """Provides convenience interfaces for working with asyncrhonously repeating tasks."""
 
+    __private_attributes__ = {
+        '_repeating_tasks': pydantic.PrivateAttr({}),
+    }
+
     def __init_subclass__(cls, **kwargs) -> None: # noqa: D105
         super().__init_subclass__(**kwargs)
 
@@ -33,17 +34,11 @@ class Mixin:
 
     def __init__(self, *args, **kwargs) -> None: # noqa: D107
         super().__init__(*args, **kwargs)
-        repeating_tasks: weakref.WeakValueDictionary[
-            str, asyncio.Task
-        ] = weakref.WeakValueDictionary()
-        _repeating_tasks_registry[self] = repeating_tasks
 
         # Start tasks for any methods decorated via `repeating`
         for method, repeat_params in self.__class__.__repeaters__.items():
             if repeat_params := getattr(method, "__repeating__", None):
-                name, duration = servo.utilities.key_paths.values_for_keys(
-                    repeat_params, "name", "duration"
-                )
+                name, duration = repeat_params["name"], repeat_params["duration"]
                 self.start_repeating_task(name, duration, method)
 
     def start_repeating_task(
@@ -68,38 +63,45 @@ class Mixin:
         async def repeating_async_fn() -> None:
             while True:
                 callable()
-                await asyncio.sleep(every / datetime.timedelta(microseconds=1))
+                await asyncio.sleep(every.total_seconds())
 
         asyncio_task = asyncio.create_task(repeating_async_fn(), name=task_name)
-        self.repeating_tasks[name] = asyncio_task
+        self._repeating_tasks[name] = asyncio_task
         return asyncio_task
 
-    def cancel_repeating_task(self, name: str) -> Optional[bool]:
+    def cancel_repeating_task(self, name: str) -> Optional[asyncio.Task]:
         """Cancel a repeating task with the given name.
 
-        Returns True if the task was cancelled, False if it was found but could not be cancelled, or None if
-        no task with the given name could be found.
+        Returns the asyncio task if one was found else None.
 
         Note that cancellation is not guaranteed (see asyncio.Task docs: https://docs.python.org/3/library/asyncio-task.html#asyncio.Task.cancel)
         """
         if task := self.repeating_tasks.get(name):
-            if task.cancelled():
-                return False
-            task.cancel()
-            return True
+            if not task.done():
+                task.cancel()
+
+            return task
 
         return None
 
+    def cancel_repeating_tasks(self) -> Dict[str, asyncio.Task]:
+        """Cancel a repeating task with the given name.
+
+        Returns a Dictionary of repeating task names to asyncio.Task objects.
+
+        Note that cancellation is not guaranteed (see asyncio.Task docs: https://docs.python.org/3/library/asyncio-task.html#asyncio.Task.cancel)
+        """
+        repeating_tasks: Dict[str, asyncio.Task] = self.repeating_tasks.copy()
+        for task in repeating_tasks.values():
+            if not task.done():
+                task.cancel()
+
+        return repeating_tasks
+
     @property
     def repeating_tasks(self) -> Dict[str, asyncio.Task]:
         """Return a dictionary of repeating tasks keyed by task name."""
-        tasks = _repeating_tasks_registry.get(self, None)
-        if tasks is None:
-            tasks = {}
-            _repeating_tasks_registry[self] = tasks
-
-        return tasks
-
+        return self._repeating_tasks
 
 def repeating(every: Every, *, name=None) -> Callable[[NoneCallable], NoneCallable]:
     """Decorate a function for repeated execution on a given duration.
